@@ -450,7 +450,7 @@ pid_t		gs_pid		= 0;
  */
 
 #ifndef KPATHSEA				/* 'H' option */
-const char	*config_file_header_path	= HEADERPATH;
+#define	config_file_header_path			header_format.default_path
 #else
 const char	*config_file_header_path	= NULL;
 #endif
@@ -1224,47 +1224,98 @@ main_search_proc P7C(char *, matpos, const char *, sp_pos,
 }
 
 static FILE *
-search P3C(const char *, path, const char *, path_var, const char *, name)
+outer_search_proc P3C(const char *, path1, const char *, path2,
+  const char *, name)
 {
-	const char	*env_path	= NULL;
-	FILE		*f;
-
-	if (path_var != NULL) {
-	    if (*name == '/') {
-		strcpy(searchpath, name);
-		return fopen(searchpath, "r");
-	    }
-	    env_path = getenv(path_var);
-	}
-	if (env_path == NULL) {
-	    env_path = path;
-	    path = NULL;
-	}
 	searchname = name;
 	searchnamelen = strlen(name);
 	for (;;) {
 	    const char *p;
 
-	    p = strchr(env_path, ':');
-	    if (p == NULL) p = env_path + strlen(env_path);
-	    if (p == env_path) {
-		if (path != NULL) {
-		    f = search(path, (char *) NULL, name);
-		    if (f != NULL) return f;
+	    p = strchr(path1, ':');
+	    if (p == NULL) p = path1 + strlen(path1);
+	    if (p == path1) {
+		if (path2 != NULL) {
+		    FILE *f;
+
+		    f = outer_search_proc(path2, (char *) NULL, name);
+		    if (f != NULL)
+			return f;
 		}
 	    }
 	    else {
 		if (setjmp(found_env))
 		    return searchfile;
 		main_search_proc(searchpath,
-		    env_path, find_dbl_slash(env_path, p), p,
+		    path1, find_dbl_slash(path1, p), p,
 		    True, &firstnode, firstnode.sp);
 	    }
-	    if (*p == '\0') return NULL;
-	    env_path = p + 1;
+	    if (*p == '\0')
+		return NULL;
+	    path1 = p + 1;
 	}
 }
 
+struct search_format {
+	const char	*default_path;
+	const char	*env_path;
+	const char	**env_var_list;
+};
+
+const char *	config_format_vars[]	= {"TEXCONFIG", NULL};
+
+struct search_format	config_format	= {
+		/* default_path */	CONFIGPATH,
+		/* env_path */		NULL,
+		/* env_var_list */	config_format_vars
+};
+
+const char *	tfm_format_vars[]	= {"GSFTOPKFONTS", "TFMFONTS",
+					   "TEXFONTS", NULL};
+
+struct search_format	tfm_format	= {
+		/* default_path */	TFMPATH,
+		/* env_path */		NULL,
+		/* env_var_list */	tfm_format_vars
+};
+
+const char *	header_format_vars[]	= {"GSFTOPKHEADERS", "TEXPSHEADERS",
+					   "PSHEADERS", NULL};
+
+struct search_format	header_format	= {
+		/* default_path */	HEADERPATH,
+		/* env_path */		NULL,
+		/* env_var_list */	header_format_vars
+};
+
+static FILE *
+search P2C(struct search_format *, sf, const char *, name)
+{
+	if (*name == '/') {
+	    strcpy(searchpath, name);
+	    return fopen(searchpath, "r");
+	}
+
+	if (sf->env_path == NULL) {
+	    const char **p = sf->env_var_list;
+	    const char *q;
+	
+	    for (;;) {
+		if (*p == NULL) {
+		    sf->env_path = sf->default_path;
+		    sf->default_path = NULL;
+		    break;
+		}
+		sf->env_path = getenv(*p);
+		if (sf->env_path != NULL)
+		    break;
+		++p;
+	    }
+	}
+
+	return outer_search_proc(sf->env_path, sf->default_path, name);
+}
+
 #endif /* not KPATHSEA */
 
 /*
@@ -2245,7 +2296,7 @@ Author of gsftopk: Paul Vojta.");
 	long_line = xmalloc(long_line_len);	/* initialize fgets_long */
 
 #ifndef KPATHSEA
-	config_file = search(CONFIGPATH, "TEXCONFIG", "config.ps");
+	config_file = search(&config_format, "config.ps");
 	if (config_file != NULL)
 	    getdefaults(config_file);
 
@@ -2266,7 +2317,7 @@ Author of gsftopk: Paul Vojta.");
 		getdefaults(config_file);
 	}
 
-	config_file = search(CONFIGPATH, "TEXCONFIG", "config.gsftopk");
+	config_file = search(&config_format, "config.gsftopk");
 	if (config_file != NULL)
 	    getdefaults(config_file);
 #else
@@ -2311,8 +2362,7 @@ Author of gsftopk: Paul Vojta.");
 		*p_tail = NULL;
 		for (p_node = p_head; p_node != NULL; p_node = p_node->next) {
 #ifndef KPATHSEA
-		    config_file = search(CONFIGPATH, "TEXCONFIG",
-		      p_node->value);
+		    config_file = search(&config_format, p_node->value);
 #else
 		    config_file = kpse_open_file(p_node->value,
 		      kpse_dvips_config_format);
@@ -2367,7 +2417,7 @@ Author of gsftopk: Paul Vojta.");
 		searchpath = kpse_find_file(q, kpse_tex_ps_header_format,true);
 		f = searchpath ? fopen(searchpath, FOPEN_R_MODE) : NULL;
 #else
-		f = search(config_file_header_path, "DVIPSHEADERS", q);
+		f = search(&header_format, q);
 #endif
 		if (f == NULL) oops("Cannot find font file %s", q);
 		/* search() also sets searchpath */
@@ -2432,8 +2482,7 @@ Author of gsftopk: Paul Vojta.");
 				    true);
 	render_ps = searchpath ? fopen (searchpath, FOPEN_R_MODE) : NULL;
 #else
-	render_ps = search(config_file_header_path, "DVIPSHEADERS",
-	    "render.ps");
+	render_ps = search(&header_format, "render.ps");
 #endif
 	if (render_ps == NULL)
 	    oops("Cannot find PS driver file \"render.ps\".");
@@ -2574,7 +2623,7 @@ Author of gsftopk: Paul Vojta.");
 #endif /* _AMIGA */
 #else /* not KPATHSEA */
 	strcpy(xfilename + fontlen, ".tfm");
-	tfm_file = search(TFMPATH, "TEXFONTS", xfilename);
+	tfm_file = search(&tfm_format, xfilename);
 #endif /* not KPATHSEA */
 	if (tfm_file == NULL) oops("Cannot find tfm file.");
 
